#include "Game.h"

Game::Game():
    stage(0)
{
    state = NONE;
    frame = 0; kills = 0;
    eventManager.add("You wake up grogilly in a dark cave, alone. Your head is pounding and you can taste blood. \nYou are hungry.");
    inventoryScreen = new TCODConsole(40, 30);
    level = new Level(entities, 70,44);
    player = new Player(eventManager, level, 1, 1);
    addEntity(player);
    level->generate();
    level->calculateFOV(player->x, player->y, 3); //hardcoded 10 radius for now.
    int tx=0, ty=0;

    for(int i = 0; i < 26; i++)
    {
        while(!level->isEmpty(tx, ty) || level->hasBlockingEntity(tx, ty) != NULL)
        {
            tx = rand()%level->w;
            ty = rand()%level->h;
        }

        addEntity(new NPC(eventManager, level, tx, ty, NPC::BAT));
    }

    ///2 stones
    for(int i = 0; i < 2; i++)
    {
        while(!level->isEmpty(tx, ty) || level->hasBlockingEntity(tx, ty) != NULL)
        {
            tx = rand()%level->w;
            ty = rand()%level->h;
        }

        addEntity(new Item(eventManager, level, tx, ty, Item::STONE));
        tx = rand()%level->w;
        ty = rand()%level->h;
    }
    for(int i = 0; i < 2; i++)
    {
        while(!level->isEmpty(tx, ty) || level->hasBlockingEntity(tx, ty) != NULL)
        {
            tx = rand()%level->w;
            ty = rand()%level->h;
        }

        addEntity(new NPC(eventManager, level, tx, ty, NPC::BEAR));
        tx = rand()%level->w;
        ty = rand()%level->h;
    }
    for(int i = 0; i < 1; i++)
    {
        while(!level->isEmpty(tx, ty) || level->hasBlockingEntity(tx, ty) != NULL)
        {
            tx = rand()%level->w;
            ty = rand()%level->h;
        }

        addEntity(new NPC(eventManager, level, tx, ty, NPC::KITTEN));
        tx = rand()%level->w;
        ty = rand()%level->h;
    }
    for(int i = 0; i < 5; i++)
    {
        while(!level->isEmpty(tx, ty) || level->hasBlockingEntity(tx, ty) != NULL)
        {
            tx = rand()%level->w;
            ty = rand()%level->h;
        }

        addEntity(new NPC(eventManager, level, tx, ty, NPC::SNAKE));
        tx = rand()%level->w;
        ty = rand()%level->h;
    }
}

Game::~Game()
{
    delete level;
    delete inventoryScreen;
}

void Game::addEntity(Entity* e)
{
    while(entities.find(e->id)!=entities.end())
    {
        //this should never happen
        e->id = Entity::CURR_ID++;
    }
    entities.insert(pair<unsigned int, Entity*>(e->id, e));
    level->tiles[e->y][e->x].entities.push_back(e);
}

void Game::drawHUD()
{
    TCODConsole::root->rect(70,0,10,45, true);
    stringstream ss("");
    ss<<player->health;
    ss<<"/";
    ss<<player->maxHealth;
    TCODConsole::root->printLeft(71, 1, TCOD_BKGND_NONE, ss.str().c_str());
    int thresh = (player->health * 10) / player->maxHealth;
    for(int i = 0; i < 10; i++)
    {
        if(i<thresh)
            TCODConsole::root->setBack(70+i, 1, TCODColor::darkRed);
    }
    ss.str("");
    ss<<"Armor: "<<player->armor;
    TCODConsole::root->printLeft(70, 3, TCOD_BKGND_NONE, ss.str().c_str());
    ss.str("");
    ss<<"Damage:"<<(player->minDmg+player->weaponAtk);
    TCODConsole::root->printLeft(70, 4, TCOD_BKGND_NONE, ss.str().c_str());
///hunger threshold display
    if(player->hunger < Player::HUNGER_HUNGRY)
    {
        TCODConsole::root->printLeft(70, 5, TCOD_BKGND_SET, "%cNot hungry%c", TCOD_COLCTRL_3, TCOD_COLCTRL_STOP);
    }
    else if(player->hunger < Player::HUNGER_VERY_HUNGRY)
    {
        TCODConsole::root->printLeft(70, 5, TCOD_BKGND_SET, "%cHungry%c", TCOD_COLCTRL_2, TCOD_COLCTRL_STOP);
    }
    else if(player->hunger < Player::HUNGER_VERY_HUNGRY)
    {
        TCODConsole::root->printLeft(70, 5, TCOD_BKGND_SET, "%cVery hungry%c", TCOD_COLCTRL_2, TCOD_COLCTRL_STOP);
    }
    else if(player->hunger < Player::HUNGER_NEAR_STARVING)
    {
        TCODConsole::root->printLeft(70, 5, TCOD_BKGND_SET, "%cFamished%c", TCOD_COLCTRL_5, TCOD_COLCTRL_STOP);
    }
    else if(player->hunger < Player::HUNGER_DEATH)
    {
        TCODConsole::root->printLeft(70, 5, TCOD_BKGND_SET, "%cStarving%c", TCOD_COLCTRL_1, TCOD_COLCTRL_STOP);
    }

    int index = 8;
    for(unsigned int i = 0; i < level->visibleEntities.size(); i++)
    {
        NPC* npc = dynamic_cast<NPC*>(level->visibleEntities[i]);
        if(npc != NULL && !npc->isPlayer)
        {

            if(rand()%6==0)
            {
                if(npc->type == NPC::KITTEN)
                {
                    eventManager.add("The kitten purrs.");
                }
            }
            TCODConsole::root->setBack(70, (index), TCODColor::lerp(TCODColor::red, TCODColor::green, npc->health / (float) npc->maxHealth));
            TCODConsole::root->printLeft(71,(index++), TCOD_BKGND_NONE, npc->name.c_str());
            if(npc->agro)
            {
                TCODConsole::root->printLeft(72,(index++), TCOD_BKGND_SET, "%cangry%c", TCOD_COLCTRL_1, TCOD_COLCTRL_STOP);
                //TCODConsole::root->setBack(70,4+index*2, TCODColor::red);
            }
            else if(npc->fleeing)
            {
                TCODConsole::root->printLeft(72,(index++), TCOD_BKGND_SET, "%cfleeing%c", TCOD_COLCTRL_2, TCOD_COLCTRL_STOP);
            }
        }

    }
    int itemIndex = 0;
    stringstream itemList("");

    Tile* tile = &(level->tiles[player->y][player->x]);
    list<Entity*>::iterator it = tile->entities.begin();

    for(;it!=tile->entities.end(); it++)
    {
        Item* item = dynamic_cast<Item*>(*it);

        if(item!=NULL)
        {
            if(itemIndex==0)
                itemList<<"You see here: ";
            if(itemIndex>0)
                itemList<<", ";
            itemList<<item->name;

            itemIndex++;
        }
    }

    ///draw items
    TCODConsole::root->rect(42, 44, 38, 5, true);
    switch(level->tiles[player->y][player->x].ascii)
    {
        case '.':
            TCODConsole::root->printLeft(42,44, TCOD_BKGND_NONE, "smooth stone floor");
            break;
        case '"':
            TCODConsole::root->printLeft(42,44, TCOD_BKGND_NONE, "scattered pebbles");
            break;
        case (16*15+7):
            TCODConsole::root->printLeft(42,44, TCOD_BKGND_NONE, "shallow pool of water");
            break;
        case 15:
            TCODConsole::root->printLeft(42,44, TCOD_BKGND_NONE, "blood-smeared floor");
            break;

    }

    TCODConsole::root->printLeftRect(42, 45, 39, 4, TCOD_BKGND_NONE, itemList.str().c_str());

    eventManager.draw();
    if(state==GAME_OVER)
    {
        inventoryScreen->clear();
        for(int i = 0; i < 40; i++)
        {
            inventoryScreen->putCharEx(i, 0, 205, TCODColor::red, TCODColor::black );
            inventoryScreen->putCharEx(i, 29, 205, TCODColor::red, TCODColor::black );
        }
        for(int i = 0; i < 30; i++)
        {
            inventoryScreen->putCharEx(0, i, 186, TCODColor::red, TCODColor::black );
            inventoryScreen->putCharEx(39, i, 186, TCODColor::red, TCODColor::black );
        }
        inventoryScreen->putCharEx(0,0,206,TCODColor::red, TCODColor::black);
        inventoryScreen->putCharEx(0,29,206,TCODColor::red, TCODColor::black);
        inventoryScreen->putCharEx(39,0,206,TCODColor::red, TCODColor::black);
        inventoryScreen->putCharEx(39,29,206,TCODColor::red, TCODColor::black);

        inventoryScreen->printLeft(4,29, TCOD_BKGND_NONE, "EXIT(ESC)");

        inventoryScreen->printLeftRect(2,2,36,26,TCOD_BKGND_NONE, "Dear Rebecca,\n\n My blackouts have been getting longer and longer, I don't know when the next time I'll be lucid is, so I'm leaving you this note so you can know the truth. I'm the one who killed Oliver, I think.\n\n I woke up last night with blood under my fingernails. It wasn't the first time either. I have another side to me... a darker side that I can't live with. By the time you find this note, I'll already be dead. \n\n I've already found a secluded cave on Mt.Herat that I can trap myself in. It is for the best that I die alone.\n\nLove,\nJames");
        TCODConsole::blit(inventoryScreen,0,0,40,30,TCODConsole::root, 20,5,1);
        TCODConsole::flush();
    }
    if(state==INVENTORY || state==DROP_ITEM)
    {
        inventoryScreen->clear();
        for(int i = 0; i < 40; i++)
        {
            inventoryScreen->putCharEx(i, 0, 205, TCODColor::lightBlue, TCODColor::black );
            inventoryScreen->putCharEx(i, 29, 205, TCODColor::lightBlue, TCODColor::black );
        }
        for(int i = 0; i < 30; i++)
        {
            inventoryScreen->putCharEx(0, i, 186, TCODColor::lightBlue, TCODColor::black );
            inventoryScreen->putCharEx(39, i, 186, TCODColor::lightBlue, TCODColor::black );
        }
        inventoryScreen->putCharEx(0,0,206,TCODColor::lightBlue, TCODColor::black);
        inventoryScreen->putCharEx(0,29,206,TCODColor::lightBlue, TCODColor::black);
        inventoryScreen->putCharEx(39,0,206,TCODColor::lightBlue, TCODColor::black);
        inventoryScreen->putCharEx(39,29,206,TCODColor::lightBlue, TCODColor::black);

        inventoryScreen->printLeft(4,29, TCOD_BKGND_NONE, "(Z) EXIT");


    }
    if(state==DROP_ITEM)
    {
        inventoryScreen->printLeft(4,0, TCOD_BKGND_NONE, "DROP ITEMS");
        for(unsigned int i = 0; i < player->items.size(); i++)
        {
            if(!player->items[i]->equipped)
            {
                inventoryScreen->putCharEx(2, i+2, 'a'+i,  TCODColor::red, TCODColor::black);
                inventoryScreen->printLeft(3, 2+i, TCOD_BKGND_NONE, ")drop");
            }
            inventoryScreen->printLeft(15, 2+i, TCOD_BKGND_NONE, player->items[i]->name.c_str());
        }
        TCODConsole::blit(inventoryScreen,0,0,40,30,TCODConsole::root,20,5,1);
        TCODConsole::flush();
    }
    if(state==INVENTORY)
    {
        inventoryScreen->printLeft(4,0, TCOD_BKGND_NONE, "INVENTORY");
        for(unsigned int i = 0; i < player->items.size(); i++)
        {
            if(player->items[i]->eatable)
            {
                inventoryScreen->putCharEx(2, i+2, 'a'+i,  TCODColor::lightRed, TCODColor::black);
                inventoryScreen->printLeft(3, 2+i, TCOD_BKGND_NONE, ")eat");
            }
            else if(player->items[i]->butcherable)
            {
                inventoryScreen->putCharEx(2, i+2, 'a'+i,  TCODColor::lightRed, TCODColor::black);
                inventoryScreen->printLeft(3, 2+i, TCOD_BKGND_NONE, ")butcher");
            }
            else if(player->items[i]->craftable)
            {
                inventoryScreen->putCharEx(2, i+2, 'a'+i,  TCODColor::lightViolet, TCODColor::black);
                inventoryScreen->printLeft(3, 2+i, TCOD_BKGND_NONE, ")craft");
            }
            else if(player->items[i]->equipped)
            {
                inventoryScreen->putCharEx(2, i+2, 'a'+i,  TCODColor::desaturatedOrange, TCODColor::black);
                inventoryScreen->printLeft(3, 2+i, TCOD_BKGND_NONE, ")unequip");
            }
            else if(player->items[i]->equipSlot>=0 && player->equipment[player->items[i]->equipSlot]==NULL)
            {
                inventoryScreen->putCharEx(2, i+2, 'a'+i,  TCODColor::orange, TCODColor::black);
                inventoryScreen->printLeft(3, 2+i, TCOD_BKGND_NONE, ")equip");
            }
            else if(player->items[i]->type == Item::NOTE)
            {
                inventoryScreen->putCharEx(2, i+2, 'a'+i,  TCODColor::white, TCODColor::black);
                inventoryScreen->printLeft(3, 2+i, TCOD_BKGND_NONE, ")read");
            }
            inventoryScreen->printLeft(15, 2+i, TCOD_BKGND_NONE, player->items[i]->name.c_str());
        }
        TCODConsole::blit(inventoryScreen,0,0,40,30,TCODConsole::root,20,5,1);
        TCODConsole::flush();
    }
}

void Game::draw()
{
    if(state==FADE_OUT)
    {
        for (int fade=255; fade >= 0; fade --)
        {
            TCODConsole::setFade(fade,TCODColor::black);
            TCODConsole::flush();
        }
        state = CREDITS;
        draw();
        return;
    }
    else if(state  == CREDITS)
    {
        TCODConsole::root->clear();
        TCODConsole::setFade(255,TCODColor::black);
        TCODConsole::flush();
        TCODConsole::root->printCenter(40, 24, TCOD_BKGND_SET,"Created by epaik for LD22");
        TCODConsole::root->printCenter(40, 26, TCOD_BKGND_SET,"Thanks for Playing");
        return;
    }
    level->draw();
    drawHUD();
}


bool Game::handleInput()
{
    TCOD_key_t key=TCODConsole::waitForKeypress(true);

    bool actionSuccess = false;
    if(state==FADE_OUT)
    {
        return true;
    }
    if(player->die || state==CREDITS)
    {
        if(key.vk == TCODK_ESCAPE)
            return false;
        return true;
    }


    if(state==INVENTORY || state==DROP_ITEM || state==GAME_OVER)
    {
        switch(key.vk)
        {
            case TCODK_ESCAPE:
                if(state==GAME_OVER)
                    state = FADE_OUT;
                else
                    state = NONE;
                break;
            case TCODK_CHAR:
            {
                if(key.c == 'z')
                    state=NONE;
                if(state==INVENTORY)
                {
                    Item* t = player->useItem(key.c-'a');
                    if(t!=NULL)
                    {
                        if(t->type == Item::NOTE)
                        {
                            state = GAME_OVER;
                        }
                    }
                }
                if(state==DROP_ITEM)
                    player->dropItem(key.c-'a');
                break;
            }
            default:
                break;
        }
        actionSuccess = true;
    }
    else
    {
    switch(key.vk)
    {
        case TCODK_ESCAPE:
            return false;

            break;
        case TCODK_UP:
            actionSuccess = player->move(Entity::UP);
            break;
        case TCODK_LEFT:
            actionSuccess = player->move(Entity::LEFT);
            break;
        case TCODK_RIGHT:
            actionSuccess = player->move(Entity::RIGHT);
            break;
        case TCODK_DOWN:
            actionSuccess = player->move(Entity::DOWN);
            break;
        case TCODK_CHAR:
            if(key.c == '.')
                actionSuccess=true;
            else if(key.c == 'h')
                actionSuccess = player->move(Entity::LEFT);
            else if(key.c == 'j')
                actionSuccess = player->move(Entity::DOWN);
            else if(key.c == 'k')
                actionSuccess = player->move(Entity::UP);
            else if(key.c == 'l')
                actionSuccess = player->move(Entity::RIGHT);
            else if(key.c == 'y')
                actionSuccess = player->move(Entity::UP_LEFT);
            else if(key.c == 'u')
                actionSuccess = player->move(Entity::UP_RIGHT);
            else if(key.c == 'b')
                actionSuccess = player->move(Entity::DOWN_LEFT);
            else if(key.c == 'n')
                actionSuccess = player->move(Entity::DOWN_RIGHT);
            else if(key.c == 'g') //grab
                actionSuccess = player->grab();
            else if(key.c == 'i' && state == NONE)
            {
                state = INVENTORY;
            }
            else if(key.c == 'd' && state == NONE)
            {
                state = DROP_ITEM;
            }

            break;

        default:
            break;
    }
    }
    if(actionSuccess)
    {
        frame++;
        if(frame>3 && stage == 0)
        {
            eventManager.add("You can't remember anything. You hear the flapping of wings."); stage++;
        }
        else if(frame>30 && stage == 1)
        {
            eventManager.add("Slowly, some memories start coming back to you. This cave feels familiar."); stage++;
        }
        else if(kills >5 && stage == 2)
        {
            eventManager.add("You feel stronger."); stage++;
        }
        else if(kills >15 && stage == 3)
        {
            eventManager.add("You feel full of life."); stage++;
        }
        else if(kills>20 && stage == 4)
        {
            eventManager.add("You can feel your primal side taking over. It feels good."); stage++;
        }
        else if(kills>30 && stage ==5)
        {
            eventManager.add("You remember why you are in this cave. You pull something out of your pocket. CHECK INVENTORY");
            player->items.push_back(new Item(eventManager, level, -1, -1, Item::NOTE));
        }
        //all other entities step.
        map<unsigned int, Entity*>::iterator it;
        for(it=entities.begin(); it!=entities.end(); it++)
        {
            it->second->step();
        }

        ///remove dead things
        map<unsigned int, Entity*>::iterator prev = entities.begin();
        for(it=entities.begin(); it!=entities.end(); it++)
        {
            if(it->second->die)
            {
                it->second->death();

                if(it->second == player)
                {
                    ///game over state

                    break;
                }
                NPC* test = dynamic_cast<NPC*>(it->second);
                if(test!=NULL)
                {
                    kills++;
                   // printf("%d kills\n", kills);
                }
                if( it==entities.begin() )
                {
                    delete it->second;
                    entities.erase(it);

                    it=entities.begin();
                    prev=entities.begin();
                }
                else
                {
                    delete it->second;
                    entities.erase(it);
                    it=prev;
                }
            }
            prev=it;
        }
        level->calculateFOV(player->x, player->y, 5); //hardcoded 10 radius for now.
    }
    return true;
}
