#include "Item.h"
#include "Player.h"

Item::Item(EventManager& eventManager, Level* level, int x, int y, int type):
    Entity( eventManager, level, x, y, false),
    eatable(false),
    butcherable(false),
    craftable(false),
    type(type),
    equipSlot(-1),
    effect(0)
{
    equipped = false;
    if(type > CORPSES_START && type < CORPSES_END)
    {
        butcherable = true;
        c = '%';
    }
     if(type > EATABLE_START && type < EATABLE_END)
     {
         c='%';
        eatable = true;
     }
     if(type > CRAFTABLE_START && type < CRAFTABLE_END)
     {
        craftable = true;
        c='!';
        if(type == STONE)
            c=7;
     }
    if(type > EQUIPABLE_START)
    {
        c='[';
        if(type<HEAD_END)
        {
            equipSlot = HEAD;
        }
        else if(type<CHEST_END)
        {
            equipSlot=CHEST;
        }
        else if(type<LEGS_END)
        {
            equipSlot=LEGS;
        }
        else if(type<WEAPONS_END)
        {
            c=')';
            equipSlot=WEAPON;
        }
    }

    setName(type);
}

Item::~Item()
{
    //dtor
}

void Item::setName(int type)
{
    switch(type)
    {
        case MYSTERY_MEAT:
            name = "mysterious hunk of meat";
            break;
        case BAT_CORPSE:
            name = "a bat corpse";
            break;
        case BAT_PELT:
            name = "bat pelt";
            break;
        case BAT_MEAT:
            name = "bat meat";
            break;
        case BAT_PELT_HAT:
            name = "bat pelt hat";
            effect=1+rand()%2;
            break;
        case BAT_PELT_SHIRT:
            name = "bat pelt shirt";
            effect=1+rand()%3;
            break;
        case BAT_PELT_PANTS:
            name = "bat pelt pants";
            effect=1+rand()%2;
            break;
         case BEAR_CORPSE:
            name = "a bear corpse";
            break;
        case BEAR_PELT:
            name = "bear pelt";
            break;
        case BEAR_MEAT:
            name = "bear meat";
            break;
        case BEAR_PELT_HAT:
            name = "bear pelt helmet";
            effect=3+rand()%2;
            break;
        case BEAR_PELT_SHIRT:
            name = "bear pelt armor";
            effect=3+rand()%3;
            break;
        case BEAR_PELT_PANTS:
            name = "bear pelt leggings";
            effect=3+rand()%2;
            break;
        case SNAKE_CORPSE:
            name = "a snake corpse";
            break;
        case SNAKE_MEAT:
            name = "snake meat";
            break;
        case BEAR_CLAW:
            name = "severed bear claw";
            effect=3+rand()%3;
            break;
        case STONE:
            name = "a small stone";
            break;
        case STONE_DAGGER:
            name = "a sharpened stone";
            effect = 1+rand()%2;
            break;
        case KITTEN_CORPSE:
            name = "a kitten corpse";
            break;
        case NOTE:
            name = "a crumpled bloody note";
            break;
        default:
            name = "an unidentified item";
            break;
    }
    if(equipSlot>=0)
    {
        stringstream add("");
        add<<name;
        add<<"(+";
        add<<effect;
        add<<")";
        name=add.str();
    }
}

void Item::butcher(Player* player) //place items into player's inventory
{
    assert(butcherable);
    switch(type)
    {
        case BAT_CORPSE:
            eventManager.add("You butcher the bat corpse.");
            if(rand()%2)
                player->items.push_back(new Item(eventManager, level, -1, -1, BAT_PELT));
            player->items.push_back(new Item(eventManager, level, -1, -1, BAT_MEAT));
            break;
        case SNAKE_CORPSE:
            eventManager.add("You butcher the snake corpse.");
            player->items.push_back(new Item(eventManager, level, -1, -1, SNAKE_MEAT));
            break;
        case BEAR_CORPSE:
            eventManager.add("You butcher the bear corpse.");
            player->items.push_back(new Item(eventManager, level, -1, -1, BEAR_PELT));
            player->items.push_back(new Item(eventManager, level, -1, -1, BEAR_MEAT));
            player->items.push_back(new Item(eventManager, level, -1, -1, BEAR_CLAW));
            break;
        case KITTEN_CORPSE:
            eventManager.add("Nothing is yielded from the kitten corpse... you monster.");
            break;
    }
}

void Item::craft(Player* player)
{
    assert(craftable);
    switch(type)
    {
        case BAT_PELT:
        {
            int r = rand()%3;
            if(r==0)
                player->items.push_back(new Item(eventManager, level, -1, -1, BAT_PELT_HAT));
            else if(r==1)
                player->items.push_back(new Item(eventManager, level, -1, -1, BAT_PELT_SHIRT));
            else if(r==2)
                player->items.push_back(new Item(eventManager, level, -1, -1, BAT_PELT_PANTS));
            break;
        }
        case BEAR_PELT:
        {
            int r = rand()%3;
            if(r==0)
                player->items.push_back(new Item(eventManager, level, -1, -1, BEAR_PELT_HAT));
            else if(r==1)
                player->items.push_back(new Item(eventManager, level, -1, -1, BEAR_PELT_SHIRT));
            else if(r==2)
                player->items.push_back(new Item(eventManager, level, -1, -1, BEAR_PELT_PANTS));
            break;
        }
        case STONE:
        {
            player->items.push_back(new Item(eventManager, level, -1, -1, STONE_DAGGER));
        }
        break;
    }
}


int Item::eat()
{
    assert(eatable);
    switch(type)
    {
        case MYSTERY_MEAT:
            eventManager.add("You eat the raw meat. It sates your hunger for now.");
            return 100;
            break;
        case BAT_MEAT:
            eventManager.add("You eat the raw bat meat. It tastes disgusting.");
            return 39;
            break;
        case SNAKE_MEAT:
            eventManager.add("You eat the raw snake meat. Not bad.");
            return 55;
            break;
        case BEAR_MEAT:
            eventManager.add("You eat the raw bear meat. You feel like a badass.");
            return 170;
    }
    return 0;
}
