#include "Level.h"
#include "Entity.h"

Level::Level(map<unsigned int, Entity*>& entities, int w, int h):
    entities(entities),
    w(w),
    h(h)
{
    tiles = new Tile*[h];
    for(int i = 0; i < h; i++)
    {
        tiles[i] = new Tile[w];
    }

    lightMap = new TCODMap::TCODMap(w, h);
}

Level::~Level()
{
    for(int i = 0; i < h; i++)
    {
        delete[] tiles[i];
    }
    delete[] tiles;
    //dtor
    delete lightMap;
}

void Level::addEntity(Entity* e)
{
    while(entities.find(e->id)!=entities.end())
    {
        //this should never happen
        e->id = Entity::CURR_ID++;
    }
    entities.insert(pair<unsigned int, Entity*>(e->id, e));
    tiles[e->y][e->x].entities.push_back(e);
}


void Level::setType(int x, int y, char type)
{
    if(x<0 || y<0 || x>=w || y>=h)
        printf("WHAT? %d %d\n", x, y);
   // assert(x>=0 && y>=0 && x<w && y<h);
    tiles[y][x].setType(type);
    //transparent, walkable
    switch(type)
    {
        case Tile::WATER_SHALLOW:
        case Tile::GROUND:
            lightMap->setProperties(x, y, true, true);
            break;
        case Tile::WALL:
            lightMap->setProperties(x, y, false, false);
            break;
        case Tile::WATER:
            lightMap->setProperties(x, y, true, false);
            break;
    }
}
bool Level::isVisible(int x, int y)
{
    return lightMap->isInFov(x, y);
}
void Level::draw()
{
    bool visible;
    bool explored;
    visibleEntities.clear();
    for(int y = 0; y < h; y++)
    {
        for(int x = 0; x < w; x++)
        {
            visible = isVisible(x, y);
            if(visible)
            {
                tiles[y][x].explored = true;
            }
            explored = tiles[y][x].explored;
            if(explored && (tiles[y][x].entities.size() == 0 || !visible))
            {
                TCODConsole::root->putCharEx(x, y, tiles[y][x].ascii, tiles[y][x].fg, tiles[y][x].bg);
            }
            else if(visible)
            {
                Entity* ent = tiles[y][x].drawEntity();
                if(ent != NULL)
                {
                    TCODConsole::root->setBack(x,y,tiles[y][x].bg);
                    TCODConsole::root->setFore(x,y,tiles[y][x].fg);

                    TCODConsole::root->setChar(x, y, ent->c);//, ent->fg, TCODColor::black);
                    visibleEntities.push_back(ent);
                }
            }
            if(!visible)
            {
                TCODConsole::root->setBack(x,y,TCODConsole::root->getBack(x, y)*0.25f);
                TCODConsole::root->setFore(x,y,TCODConsole::root->getFore(x, y)*0.25f);
            }
        }
    }
}
void Level::recurse(TCODBsp* node)
{
    int startx = node->x;
    int starty = node->y;
    int endx = node->x + node->w;
    int endy = node->y + node->h;
//    while(rand()%2==0)
//        startx++;
//    while(rand()%4==0)
//        starty++;
//    while(rand()%2==0)
//        endx--;
//    while(rand()%4==0)
//        endy--;
    for(int x = startx; x < endx; x++)
    {
        for(int y = starty; y < endy; y++)
        {
            if(x==startx || y == starty || x == endx || y == endy )
            {
                setType(x, y, Tile::WALL);
            }
            else
            {
//                if((x==node->x+1 || y == node->y+1 || x == node->x+node->w-1 || y == node->y + node->h -1))
//                {
//                    int count = (tiles[y][x-1].type == Tile::WALL ? 1 : 0) +
//                                (tiles[y][x+1].type == Tile::WALL ? 1 : 0) +
//                                (tiles[y+1][x].type == Tile::WALL ? 1 : 0) +
//                                (tiles[y-1][x].type == Tile::WALL ? 1 : 0);
//                    if(rand()%9 < count)
//                    {
//                        setType(x, y, Tile::WALL);
//                        continue;
//                    }
//                }
                setType(x, y, Tile::GROUND);
            }
        }
    }

    if(!node->isLeaf())
    {
        recurse(node->getRight());
        recurse(node->getLeft());

        if(!node->horizontal)
        {
            int pos = rand()%(node->h-2) + 1 + node->y;
            int i = node->position;
            while(tiles[pos][i].type == Tile::WALL && i < w)
            {
                setType(i, pos, Tile::GROUND);
                i++;
            }
            i = node->position-1;
            while(tiles[pos][i].type == Tile::WALL && i > 0)
            {
                setType(i, pos, Tile::GROUND);
                i--;
            }
        }
        else
        {
            int pos = rand()%(node->w-2) + 1 + node->x;
            int i = node->position;
            while(tiles[i][pos].type == Tile::WALL && i < h)
            {
                setType(pos, i, Tile::GROUND);
                i++;
            }
            i = node->position-1;
            while(tiles[i][pos].type == Tile::WALL && i > 0)
            {
                setType(pos, i, Tile::GROUND);
                i--;
            }
        }
    }
}
void Level::generate()
{
    bsp = new TCODBsp(0, 0, w, h);
    bsp->splitRecursive(NULL,7, 4,4,2.5f, 1.9f); //randomizer, recurse levels, minimum size(x, y), max ratios
    recurse(bsp);

    for(int y = 0; y < h; y++)
    {
        setType(w-1, y, Tile::WALL);
    }
    for(int x = 0; x < w; x++)
    {
        setType(x, h-1, Tile::WALL);
    }

    //done with main phase, now lets place some water sources.
    for(int i = 0; i < 3; i++) ///three water sources
    {
        int randPosX = rand()%w;
        int randPosY = rand()%h;

        for(int y = 0; y < h; y++)
        {
            for(int x = 0; x < w; x++)
            {
                if(sqrt(pow(randPosX-x, 2) + pow(randPosY-y, 2)) < (rand()%3+2))
                {
                    if(tiles[y][x].type == Tile::WALL)
                        setType(x, y, Tile::WATER);
                    else if(tiles[y][x].type == Tile::GROUND)
                        setType(x, y, Tile::WATER_SHALLOW);
                }
            }
        }
    }
}
void Level::calculateFOV(int x, int y, int r)
{
    lightMap->computeFov(x, y, r);
}
bool Level::isEmpty(int x, int y)
{
    if(tiles[y][x].type == Tile::GROUND ||
       tiles[y][x].type == Tile::WATER_SHALLOW)
    {
        return true;
    }
    return false;
}

Entity* Level::hasBlockingEntity(int x, int y)
{
    return tiles[y][x].hasBlockingEntity();
}

float Level::distance(int ax, int ay, int bx, int by)
{
    return sqrt(pow(ax-bx, 2)+pow(ay-by, 2));
}
