#include "NPC.h"

NPC::NPC(EventManager& eventManager, Level* level, int x, int y, char type):
    Entity(eventManager, level, x, y),
    type(type),
    target(NULL),
    isPlayer(false),
    aggressive(false),
    agro(false),
    fleeing(false),
    fleeTimer(0),
    wounded(false),
    turn(0)
{
    armor = 1;
    weaponAtk = 1;
    friendly = false;
    switch(type)
    {
        case BAT:
            health=maxHealth=5;
            speed = 14;
            agroRadius = 6;
            minDmg = 0;
            fleeThreshold = 1;
            woundedThreshold = 3;
            dmgRange = 2; //0-2 damage
            hit = 8;
            name = "bat";
            c = 'b';
            fg = TCODColor::white;
            break;
        case SNAKE:
            health=maxHealth=7;
            speed = 11;
            aggressive = true;
            agroRadius = 4;
            minDmg = 2;
            fleeThreshold = 2;
            woundedThreshold = 3;
            dmgRange = 2; //0-2 damage
            hit = 9;
            name = "snake";
            c = 's';
            fg = TCODColor::white;
            break;
        case BEAR:
            health=maxHealth=20;
            speed = 8;
            aggressive = true;
            agroRadius = 3;
            minDmg = 5;
            fleeThreshold = 4;
            woundedThreshold = 5;
            dmgRange = 3; //0-2 damage
            hit = 7;
            name = "bear";
            c = 'B';
            fg = TCODColor::white;
            break;

        case KITTEN:
            health=maxHealth=1;
            speed = 4;
            agroRadius = 0;
            minDmg = 0;
            fleeThreshold = 1;
            woundedThreshold = 3;
            dmgRange = 0; //0-2 damage
            hit = 0;
            name = "kitten";
            c = 'k';
            fg = TCODColor::pink;
            break;
    }

}

NPC::~NPC()
{
    //dtor
}
void NPC::attack(Entity* e)
{
    NPC* other = dynamic_cast<NPC*>(e);
    if((e->friendly && friendly) || (!e->friendly && !friendly))
        return; //cannot attack own faction
    agroTimer = 0;
    if(other->agro == false)
    {
        other->agro = true;
        other->target = this;
    }
    if(rand()%10 <hit)
    {
        int dmg = rand()%dmgRange + minDmg + (rand()%weaponAtk) - (rand()%other->armor);
        if(dmg<0)
            dmg = 0;
        //successful hit!
        other->health-=dmg;
        if(other->health<=0)
            other->die = true;

        eventManager.addCombatText(this, other, dmg);
    }
    else
    {
        eventManager.addCombatText(this, other, -1);
    }
}
void NPC::action()
{
    //MOVE PHASE
    if(fleeing)
    {
        int tx = x;
        int ty = y;
        //move away from enemy
        if(target->x < x)
            tx++;
        else if(target->x > x)
            tx--;
        if(target->y < y)
            ty++;
        else if(target->y > y)
            ty--;
        move(tx, ty);
        //increment timer
        fleeTimer++;
        if(fleeTimer == 10)
        {
            fleeTimer = 0;
            fleeing = false;
            agro = false;
            target = NULL;
        }
    }
    else
    {
        //scan for enemies
        if(!agro && aggressive)
        {
            for(int ax = x-agroRadius; ax <= x+agroRadius; ax++)
            {
                for(int ay = y-agroRadius; ay <= y+agroRadius; ay++)
                {
                    if(ax<0 || ay<0 || ax>=level->w || ay>=level->h)
                        continue;
                    if(level->distance(ax, ay, x, y) > agroRadius)
                        continue;
                    if(!level->isVisible(ax, ay))
                        continue; ///only visible things

                    Entity* check;
                    check = level->hasBlockingEntity(ax,ay);
                    if(check!=NULL)
                    {
                        //NPC* other = dynamic_cast<NPC*>(check);

                        if((friendly && check->friendly) ||
                            (!(friendly) && !(check->friendly)))
                            continue;
                        //attack!
                        target = check;
                        agro = true;
                        break;
                    }
                }
                if(agro)
                    break;
            }
        }
        //move
        if(agro)
        {
            TCODPath path(level->lightMap);
            path.compute(x, y, target->x, target->y);
            int nx, ny;
            path.get(0, &nx, &ny);
            if(!move(nx, ny))
            {
                //printf("%s: path blocked! :(\n", name.c_str());
            }
        }
        else
        {
            //move randomly
            int tries = 0;
            int dir = rand()%NUM_DIRECTIONS;
            while(!move(dir) && tries < NUM_DIRECTIONS)
            {
                tries++;
                dir++;
                dir=dir%NUM_DIRECTIONS;
            }
        }
    }

    if(!fleeing && health<=fleeThreshold && agro)
    {
        eventManager.add("The "+name+" is fleeing!");
        fleeing = true;
        agro = false;
    }
    if(health<=woundedThreshold)
    {
        wounded = true;
    }
}

void NPC::death()
{
    switch(type)
    {
        case BAT:
            level->addEntity(new Item(eventManager, level, x, y, Item::BAT_CORPSE));
            break;
        case SNAKE:
            level->addEntity(new Item(eventManager, level, x, y, Item::SNAKE_CORPSE));
            break;
        case BEAR:
            level->addEntity(new Item(eventManager, level, x, y, Item::BEAR_CORPSE));
            break;
        case KITTEN:
            level->addEntity(new Item(eventManager, level, x, y, Item::KITTEN_CORPSE));
            break;
    }
    if(isPlayer)
        eventManager.add("You have died alone...");
}


void NPC::step() //called each time player does an action
{
    if(agro)
    {
        agroTimer++;
        if(agroTimer > 10)
        {
            target = NULL;
            agro = false;
        }
    }
    if(die)
        return;
    turn+=speed;
    while(turn>10)
    {
        turn-=10;
        action();
    }
}
