#include "Player.h"

Player::Player(EventManager& eventManager, Level* level, int x, int y):
    NPC(eventManager, level, x, y)
{
    hunger = 50;
    thirst = 0;
    stepCounter = 0;
    isPlayer = true;
    friendly=true;
    c = '@';
   fg = TCODColor::TCODColor(150,150,250);

    health = maxHealth=20;
    speed = 10;
    minDmg = 1;
    dmgRange = 3;
    woundedThreshold = 5;
    hit = 7;
    name = "You";
    equipment = new Item*[Item::NUM_EQUIP_SLOTS];
    for(int i = 0; i < Item::NUM_EQUIP_SLOTS; i++)
    {
        equipment[i] = NULL;
    }
    items.push_back(new Item(eventManager, level, -1, -1, Item::MYSTERY_MEAT));
   // items.push_back(new Item(eventManager, level, -1, -1, Item::NOTE));
}

Player::~Player()
{
    for(unsigned int i = 0; i < items.size(); i++)
    {
        delete items[i];
    }
}

void Player::step()
{
    hunger++;
    thirst++;

    stepCounter++;
    if(stepCounter>REGEN_STEP)
    {
        stepCounter = 0;
        if(health<maxHealth && hunger < HUNGER_NEAR_STARVING)
            health++;
    }
    if(hunger>=HUNGER_DEATH)
    {
        die = true;
    }
}
void Player::dropItem(unsigned int index)
{
    if(index>=items.size())
        return;
    if(items[index]->equipped)
        return;
    if(items[index]->type == Item::NOTE)
    {
        eventManager.add("You can't drop that.");
        return;
    }
    level->tiles[y][x].add(items[index]);
    items[index]->y = y;
    items[index]->x = x;
    items.erase(items.begin()+index);
}
Item* Player::useItem(unsigned int index)
{
    if(index>=items.size())
        return NULL;
    if(items[index]->type == Item::NOTE)
    {

        return items[index];
    }
    if(items[index]->butcherable)
    {
        items[index]->butcher(this);
        //delete item here
        level->entities.erase(items[index]->id);
        delete items[index];
        items.erase(items.begin() + index);

    }
    else if(items[index]->eatable)
    {
        hunger-=items[index]->eat();
        if(hunger<0)
            hunger = 0;
        level->entities.erase(items[index]->id);
        delete items[index];
        items.erase(items.begin() + index);
    }
    else if(items[index]->craftable)
    {
        items[index]->craft(this);
        //delete item here
        level->entities.erase(items[index]->id);
        delete items[index];
        items.erase(items.begin() + index);
    }
    else if(items[index]->equipSlot >= 0)
    {
        if(equipment[items[index]->equipSlot] == NULL)
        {
            equipment[items[index]->equipSlot] = items[index];
            items[index]->equipped = true;

            if(items[index]->type < Item::WEAPONS_START)
            {
                armor+=items[index]->effect;
            }
            else
            {
                weaponAtk+=items[index]->effect;
            }
        }
        else if(items[index]->equipped)
        {
            equipment[items[index]->equipSlot] = NULL;
            items[index]->equipped = false;
            if(items[index]->type < Item::WEAPONS_START)
            {
                armor-=items[index]->effect;
            }
            else
            {
                weaponAtk-=items[index]->effect;
            }
        }
    }
   // items[index]->use();
   return NULL;
}
bool Player::grab()
{
    Tile* tile = &(level->tiles[y][x]);
    list<Entity*>::iterator it = tile->entities.begin();

    for(;it!=tile->entities.end(); it++)
    {
        Item* item = dynamic_cast<Item*>(*it);

        if(item!=NULL)
        {
            item->removeFromMap();
            items.push_back(item);
            eventManager.add("You pick up "+item->name+".");
            return true;
        }
    }
    return false;
}
