#include "Tile.h"
#include "Entity.h"

Tile::Tile():
    type(GROUND)
{
    ascii = 0;
    explored = false;
}

Tile::~Tile()
{
    list<Entity*>::iterator it;
    for(it = entities.begin(); it != entities.end(); it++)
    {
        delete (*it);
    }
}

void Tile::setType(char t)
{
    type = t;
    switch(type)
    {
        case GROUND:
            if(rand()%30==0)
            {
                ascii = 15;
                fg = TCODColor::darkRed;//TCODColor(250, 200, 200);
            }
            else if(rand()%5==0)
            {
                ascii = '"';
                fg = TCODColor::TCODColor(200, 200, 200);
            }
            else
            {
                ascii = '.';
                fg = TCODColor::TCODColor(200, 200, 200);
            }
            bg = TCODColor::black + TCODColor::TCODColor(rand()%10, rand()%10, rand()%10);


            break;
        case WALL:
            ascii = '#';
            bg = TCODColor::TCODColor(180+rand()%20, 180+rand()%20, 200+rand()%50);
            fg = TCODColor::black;
            break;
        case WATER:
            ascii = 16*15+7;
            bg = TCODColor::TCODColor(55, 55, 220+rand()%30);
            fg = TCODColor::TCODColor(55, 55, 155);
            break;
        case WATER_SHALLOW:
            ascii = 16*15+7;
            bg = TCODColor::TCODColor(175, 175, 200+rand()%50);
            fg = TCODColor::TCODColor(55, 55, 155);
            break;
    }
}

bool Tile::remove(unsigned int id)
{
    list<Entity*>::iterator it;
    for(it = entities.begin(); it != entities.end(); it++)
    {
        if((*it)->id == id)
        {
            entities.erase(it);
            return true;
        }
    }
    return false;
}

void Tile::add(Entity* e)
{
    entities.push_back(e);
}

Entity* Tile::hasBlockingEntity()
{
    list<Entity*>::iterator it;
    for(it = entities.begin(); it != entities.end(); it++)
    {
        if((*it)->blocking)
        {
            return (*it);
        }
    }
    return NULL;
}

Entity* Tile::drawEntity()
{
    Entity* toReturn = hasBlockingEntity();
    if(toReturn != NULL)
        return toReturn;
    if(entities.size() > 0)
        return entities.front();
    return NULL;
}
